open Printf
open Atom
open Types
open Terms
open Typerr
open Typematch
open Coercions
open Print

(* ------------------------------------------------------------------------- *)

(* Generating fresh atoms. *)

let mkatom name sort = Atom.fresh (Identifier.mk name sort)
let mkvar name = mkatom name Syntax.term_sort

(* for types we can create a new identifier
    or reuse an identifier from a type context *)
type name = New of string | Same of ftype_context
let mktyvar from =
  match from with 
  | New name -> mkatom name Syntax.type_sort
  | Same cty -> fresh (hint cty)

(* we usually need both an atom for binding, and a free var for terms *)
let atom_var name = let x = mkvar name in x, var x 
let atom_tyvar from = let a = mktyvar from in a, TyFreeVar a 


(* ------------------------------------------------------------------------- *)

(* Translating coercions:

   While the elaboration of coercions is always called externally with the
   domain of the coercion, it may may internally be called with either the
   domain or the codomain of the coercion, depending on polarity of the
   occurrence of the coercion. 

   The polarity is used to determined the full type of the coercion, i.e. 
   the codomain when [polarity] is [true] or the domain otherwise.

   However, once the full type has been determined, the polarity does
   not change the elaboration.  Therefore, much of the code does not
   depend on polarity and can be shared. This is why we have only one
   definition of coercion with any a boolean argument for the polarity
   rather than two recursive functions for each of the polarities.
*)

let rec coercion polarity p loc co ty = 

  (* First, determine the new current location *)
  let loc, co = Error.override loc co in

  (* Then, we examine the coercion. In each case, we often distinguish
     two subcases according to the polarity *)
  match co with
       | _ ->
       failwith "ELABORATION IS NOT IMPLEMENTED YET!" (* do something here! *)

let coercion p loc co ty = coercion true p loc co ty 

